/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
* Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 * 
* http://www.eclipse.org/legal/epl-v10.html
 * 
* Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project;

import java.util.HashSet;
import java.util.Set;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.Node;

/**
 * Provides nodes for a stock project. 
 * Stock providers like Crop will provide their own NodeFactory to produce nodes.
 *
 * @author Gerben Feenstra
 */
@NodeFactory.Registration(projectType = StockProject.STOCK_NODEFACTORY_ID)
public class StockNodeFactory implements NodeFactory {

    @Override
    public NodeList createNodes(Project project) {
            
        //Get the stock directory, creating if deleted
        FileObject projectDirectory = project.getProjectDirectory();
        FileObject stockSubFolder = projectDirectory.getFileObject(StockProject.STOCK_SUBFOLDER);
        DataFolder stockSubDataFolder = DataFolder.findFolder(stockSubFolder);
        
        Set<Node> nodes = new HashSet<>();
        nodes.add(new StockSubNode(stockSubDataFolder.getNodeDelegate()));
        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }
}