/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.project;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ProjectFactory;
import org.netbeans.spi.project.ProjectState;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;

/**
 *
 * @author Timon Veenstra
 */
@org.openide.util.lookup.ServiceProvider(service = ProjectFactory.class)
public class StockProjectFactory implements ProjectFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.stock.project.StockProjectFactory");
    public static final String ICON = "nl/cloudfarming/client/stock/project/box.png";
    public static final String STOCK_FOLDER = "stock";

    //Specifies when a project is a project, i.e.,
    //if the project directory "texts" is present:
    @Override
    public boolean isProject(FileObject projectDirectory) {
        try {
            return getStockFolder(projectDirectory,false) != null;
        } catch (IOException ex) {
            LOGGER.log(Level.WARNING,"Exception occurred while trying to determine if folder {1} is a project {0}",new Object[]{ex.getMessage(),projectDirectory.getPath()});
            return false;
        }
    }

    //Specifies when the project will be opened, i.e.,
    //if the project exists:
    @Override
    public Project loadProject(FileObject dir, ProjectState state) throws IOException {
        StockProject project = null;
        if (isProject(dir)) {
            // load stock data here and set it on the project.
//            Stock stock = stockManager.load(dir.getFileObject(STOCK_FILE));
            project = new StockProject(dir, state);
        }
        return project;
    }

    @Override
    public void saveProject(final Project project) throws IOException, ClassCastException {
        FileObject projectRoot = project.getProjectDirectory();
        // save stock data here 
    }
    
    private FileObject getStockFolder(FileObject projectFolder, boolean create) throws IOException{
        FileObject stockFolder = projectFolder.getFileObject(STOCK_FOLDER);
        if (stockFolder == null && create ){
            LOGGER.log(Level.FINEST,"Stock folder {0} did not exist, creating now.",STOCK_FOLDER);
            stockFolder = projectFolder.createFolder(STOCK_FOLDER);
        }
        if (stockFolder != null && !stockFolder.isFolder()){
            LOGGER.log(Level.SEVERE,"stock folder {0} is not a folder, probably a file. Project will not be recognized!",STOCK_FOLDER);
            stockFolder = null;
        }
        return stockFolder;
    }
}
