/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.project;

import java.awt.Component;
import java.io.*;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.LinkedHashSet;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.spi.project.ui.support.ProjectChooser;
import org.netbeans.spi.project.ui.templates.support.Templates;
import org.openide.WizardDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.xml.XMLUtil;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class StockProjectWizardIterator implements WizardDescriptor./*Progress*/InstantiatingIterator {

    private int index;
    private WizardDescriptor.Panel[] panels;
    private WizardDescriptor wiz;


    public static StockProjectWizardIterator createIterator() {
        return new StockProjectWizardIterator();
    }

    private WizardDescriptor.Panel[] createPanels() {
        return new WizardDescriptor.Panel[]{
                    new StockProjectWizardPanel(),};
    }

    private String[] createSteps() {
        return new String[]{
                    NbBundle.getMessage(StockProjectWizardIterator.class, "LBL_CreateProjectStep")
                };
    }

    @Override
    public Set/*<FileObject>*/ instantiate(/*ProgressHandle handle*/) throws IOException {
        Set<FileObject> resultSet = new LinkedHashSet<>();
        File dirF = FileUtil.normalizeFile((File) wiz.getProperty("projdir"));
        dirF.mkdirs();

        FileObject template = Templates.getTemplate(wiz);
        FileObject dir = FileUtil.toFileObject(dirF);
        unZipFile(template.getInputStream(), dir);
       
        
        // Always open top dir as a project:
        resultSet.add(dir);
        // Look for nested projects to open as well:
        Enumeration<? extends FileObject> e = dir.getFolders(true);
        while (e.hasMoreElements()) {
            FileObject subfolder = e.nextElement();
            if (ProjectManager.getDefault().isProject(subfolder)) {
                resultSet.add(subfolder);
            }
        }

        File parent = dirF.getParentFile();
        if (parent != null && parent.exists()) {
            ProjectChooser.setProjectsFolder(parent);
        }

        return resultSet;
    }

    
    /**
     * Method to load a few default crops into a project
     *
     * @param dir the dir to place the crops
     * @throws IOException
     */
    @NbBundle.Messages({
        "crop.genus.triticum_aestivum.winter.name=Winter wheat", 
        "crop.race.triticum_aestivum.alcyon=Alcyon", 
        "crop.genus.triticum_aestivum.spring.name=Spring wheat", 
        "crop.race.triticum_aestivum.alexandria=Alexandria", 
        "crop.genus=Class 1",
        "cropsample.crop.source_id=Sample crop"

    })
    // this does not belong here, should be in stock-crop module removed to fix module dependency
//    private void generateCrops(FileObject dir) throws IOException {
//        final String CROPFILE_WINTER_WHEAT = "crops.crp";
//
//        final String GENUS = NbBundle.getMessage(StockProjectWizardIterator.class, "crop.genus");
//        final String WINTER_WHEAT_CROP = NbBundle.getMessage(StockProjectWizardIterator.class, "crop.genus.triticum_aestivum.winter.name");
//        final String WINTER_WHEAT_TRITICUM_AESTIVUM_ALCUON = NbBundle.getMessage(StockProjectWizardIterator.class, "crop.race.triticum_aestivum.alcyon");
//
//        final String SPRING_WHEAT_CROP = NbBundle.getMessage(StockProjectWizardIterator.class, "crop.genus.triticum_aestivum.spring.name");
//        final String SPRING_WHEAT_TRITICUM_AESTIVUM_ALCUON = NbBundle.getMessage(StockProjectWizardIterator.class, "crop.race.triticum_aestivum.alexandria");
//
//
//        try (ObjectOutputStream oos = new ObjectOutputStream(dir.createAndOpen(CROPFILE_WINTER_WHEAT));) {
//            CropsStorage cropsStorage = new CropsStorage();
//
//            CropDefinition crop = new CropDefinition(NbBundle.getMessage(StockProjectWizardIterator.class, "cropsample.crop.source_id"));
//            crop.setName(WINTER_WHEAT_CROP);
//            Variety winterWheatTriticum = new Variety();
//            winterWheatTriticum.setName(WINTER_WHEAT_TRITICUM_AESTIVUM_ALCUON);
//            Genus genus = new Genus();
//            genus.setName(GENUS);
//            crop.addGenus(genus);
//            crop.addVariety(winterWheatTriticum);
//
//
//            CropDefinition crop2 = new CropDefinition(NbBundle.getMessage(StockProjectWizardIterator.class, "cropsample.crop.source_id"));
//            crop2.setName(SPRING_WHEAT_CROP);
//            Variety springWheatTriticum = new Variety();
//            springWheatTriticum.setName(SPRING_WHEAT_TRITICUM_AESTIVUM_ALCUON);
//            crop2.addGenus(genus);
//            crop2.addVariety(springWheatTriticum);
//
//            cropsStorage.addCrop(crop2);
//            cropsStorage.addCrop(crop);
//            
//
//            oos.writeObject(cropsStorage);
//        }
//    }

    @Override
    public void initialize(WizardDescriptor wiz) {
        this.wiz = wiz;
        index = 0;
        panels = createPanels();
        // Make sure list of steps is accurate.
        String[] steps = createSteps();
        for (int i = 0; i < panels.length; i++) {
            Component c = panels[i].getComponent();
            if (steps[i] == null) {
                // Default step name to component name of panel.
                // Mainly useful for getting the name of the target
                // chooser to appear in the list of steps.
                steps[i] = c.getName();
            }
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                // Step #.
                // TODO if using org.openide.dialogs >= 7.8, can use WizardDescriptor.PROP_*:
                jc.putClientProperty("WizardPanel_contentSelectedIndex", new Integer(i));
                // Step name (actually the whole list for reference).
                jc.putClientProperty("WizardPanel_contentData", steps);
            }
        }
    }

    @Override
    public void uninitialize(WizardDescriptor wiz) {
        this.wiz.putProperty("projdir", null);
        this.wiz.putProperty("name", null);
        this.wiz = null;
        panels = null;
    }

    @Override
    public String name() {
        return MessageFormat.format("{0} of {1}",
                new Object[]{new Integer(index + 1), new Integer(panels.length)});
    }

    @Override
    public boolean hasNext() {
        return index < panels.length - 1;
    }

    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    @Override
    public void nextPanel() {
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
    }

    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
    }

    @Override
    public WizardDescriptor.Panel current() {
        return panels[index];
    }

    // If nothing unusual changes in the middle of the wizard, simply:
    @Override
    public final void addChangeListener(ChangeListener l) {
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
    }

    private static void unZipFile(InputStream source, FileObject projectRoot) throws IOException {
        try {
            ZipInputStream str = new ZipInputStream(source);
            ZipEntry entry;
            while ((entry = str.getNextEntry()) != null) {
                if (entry.isDirectory()) {
                    FileUtil.createFolder(projectRoot, entry.getName());
                } else {
                    FileObject fo = FileUtil.createData(projectRoot, entry.getName());
                    if ("nbproject/project.xml".equals(entry.getName())) {
                        // Special handling for setting name of Ant-based projects; customize as needed:
                        filterProjectXML(fo, str, projectRoot.getName());
                    } else {
                        writeFile(str, fo);
                    }
                }
            }
        } finally {
            source.close();
        }
    }

    private static void writeFile(ZipInputStream str, FileObject fo) throws IOException {
        try (OutputStream out = fo.getOutputStream()) {
            FileUtil.copy(str, out);
        }
    }

    private static void filterProjectXML(FileObject fo, ZipInputStream str, String name) throws IOException {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            FileUtil.copy(str, baos);
            Document doc = XMLUtil.parse(new InputSource(new ByteArrayInputStream(baos.toByteArray())), false, false, null, null);
            NodeList nl = doc.getDocumentElement().getElementsByTagName("name");
            if (nl != null) {
                for (int i = 0; i < nl.getLength(); i++) {
                    Element el = (Element) nl.item(i);
                    if (el.getParentNode() != null && "data".equals(el.getParentNode().getNodeName())) {
                        NodeList nl2 = el.getChildNodes();
                        if (nl2.getLength() > 0) {
                            nl2.item(0).setNodeValue(name);
                        }
                        break;
                    }
                }
            }
            try (OutputStream out = fo.getOutputStream()) {
                XMLUtil.write(doc, out, "UTF-8");
            }
        } catch (IOException | SAXException | DOMException ex) {
            Exceptions.printStackTrace(ex);
            writeFile(str, fo);
        }

    }
}
